
/*  A Bison parser, made from mwmparse.y
    by GNU Bison version 1.28  */

#define YYBISON 1  /* Identify Bison output.  */

#define	ALT_TOK	257
#define	APP_TOK	258
#define	BACK_TOK	259
#define	BORDER_TOK	260
#define	BTN1_CLICK2_TOK	261
#define	BTN1_CLICK_TOK	262
#define	BTN1_DOWN_TOK	263
#define	BTN1_UP_TOK	264
#define	BTN2_CLICK2_TOK	265
#define	BTN2_CLICK_TOK	266
#define	BTN2_DOWN_TOK	267
#define	BTN2_UP_TOK	268
#define	BTN3_CLICK2_TOK	269
#define	BTN3_CLICK_TOK	270
#define	BTN3_DOWN_TOK	271
#define	BTN3_UP_TOK	272
#define	BTN4_CLICK2_TOK	273
#define	BTN4_CLICK_TOK	274
#define	BTN4_DOWN_TOK	275
#define	BTN4_UP_TOK	276
#define	BTN5_CLICK2_TOK	277
#define	BTN5_CLICK_TOK	278
#define	BTN5_DOWN_TOK	279
#define	BTN5_UP_TOK	280
#define	BUTTONS_TOK	281
#define	CTRL_TOK	282
#define	FBEEP_TOK	283
#define	FCIRCLE_DOWN_TOK	284
#define	FCIRCLE_UP_TOK	285
#define	FEXEC_TOK	286
#define	FFOCUS_COLOR_TOK	287
#define	FFOCUS_KEY_TOK	288
#define	FKILL_TOK	289
#define	FLOWER_TOK	290
#define	FMAXIMIZE_TOK	291
#define	FMENU_TOK	292
#define	FMINIMIZE_TOK	293
#define	FMOVE_TOK	294
#define	FNEXT_CMAP_TOK	295
#define	FNEXT_KEY_TOK	296
#define	FNOP_TOK	297
#define	FNORMALIZE_TOK	298
#define	FNORMANDRAISE_TOK	299
#define	FPACK_ICONS_TOK	300
#define	FPASS_KEYS_TOK	301
#define	FPOST_WMENU_TOK	302
#define	FPREV_CMAP_TOK	303
#define	FPREV_KEY_TOK	304
#define	FQUIT_MWM_TOK	305
#define	FRAISE_LOWER_TOK	306
#define	FRAISE_TOK	307
#define	FRAME_TOK	308
#define	FREE_FAMILY_TOK	309
#define	FREFRESH_TOK	310
#define	FREFRESH_WIN_TOK	311
#define	FRESIZE_TOK	312
#define	FRESTART_TOK	313
#define	FRESTOREANDRAISE_TOK	314
#define	FRESTORE_TOK	315
#define	FSCREEN_TOK	316
#define	FSEND_MSG_TOK	317
#define	FSEPARATOR_TOK	318
#define	FSET_BEHAVIOR_TOK	319
#define	FTITLE_TOK	320
#define	FWINDOWLIST_TOK	321
#define	FDESK_TOK	322
#define	FTOGGLE_PAGE_TOK	323
#define	FGOTO_PAGE_TOK	324
#define	ICON_TOK	325
#define	KEY_TOK	326
#define	KEYS_TOK	327
#define	LOCK_TOK	328
#define	MENU_TOK	329
#define	MENUB_TOK	330
#define	MINIMIZEB_TOK	331
#define	MAXIMIZEB_TOK	332
#define	MOD1_TOK	333
#define	MOD2_TOK	334
#define	MOD3_TOK	335
#define	MOD4_TOK	336
#define	MOD5_TOK	337
#define	NEXT_TOK	338
#define	PREV_TOK	339
#define	ROOT_TOK	340
#define	SHIFT_TOK	341
#define	TITLE_TOK	342
#define	TRANSIENT_TOK	343
#define	WINDOW_TOK	344
#define	WITHIN_TOK	345
#define	STRING_TOK	346

#line 1 "mwmparse.y"

/**
 *
 * $Id: mwmparse.y,v 1.31 2001/08/02 19:45:28 amai Exp $
 *
 * Copyright (C) 1995 Free Software Foundation, Inc.
 * Copyright (C) 1995-2001 LessTif Development Team
 *
 * This file is part of the GNU LessTif Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/

#include "LTconfig.h"

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_PWD_H
#include <pwd.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include <X11/Xfuncs.h>
#ifdef __EMX__
#include <X11/Xlibint.h>    /* for __XOS2RedirRoot() */
#endif

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>

#include "mwm.h"


/*
 * this is OS dependent, but this should catch most
 */
#define MAX_PATH_LEN	2048

extern int yydebug;
extern int yylex(void);

static int num_items;
static MenuRoot *cur_menu;
static ScreenInfo *pscr;
static Boolean skip_test;
static Boolean button_bind_match;
static Boolean button_bind_found;
static Boolean key_bind_match;
static Boolean key_bind_found;
extern int	verbose;

/*
 * this must be here for the parser
 */
static int lineno;


#line 238 "mwmparse.y"
typedef union {
    char	*string;
    int		number;
    KeySym	key;
    struct {
	int func;
	char *arg;
    } function;
    struct {
	int type;
	char *string;
    } label;
    long	modifiers;
    struct {
	int button;
	int event;
	int count;
	int modifiers;
    } button;
    struct {
	int modifiers;
	KeySym key;
    } hotkey;
} YYSTYPE;
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		176
#define	YYFLAG		-32768
#define	YYNTBASE	102

#define YYTRANSLATE(x) ((unsigned)(x) <= 346 ? yytranslate[x] : 131)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,   100,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,   101,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,    96,
     2,    97,     2,    98,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,    99,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    93,    95,    94,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
    17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
    27,    28,    29,    30,    31,    32,    33,    34,    35,    36,
    37,    38,    39,    40,    41,    42,    43,    44,    45,    46,
    47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
    57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
    67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
    77,    78,    79,    80,    81,    82,    83,    84,    85,    86,
    87,    88,    89,    90,    91,    92
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     3,     4,     6,     8,    10,    11,    18,    21,    23,
    24,    28,    29,    36,    39,    41,    42,    46,    47,    54,
    57,    59,    60,    65,    67,    68,    72,    74,    76,    78,
    80,    82,    84,    86,    88,    90,    92,    94,    96,   101,
   105,   111,   116,   119,   121,   123,   125,   127,   129,   131,
   133,   135,   137,   139,   141,   143,   145,   147,   149,   151,
   153,   155,   157,   159,   161,   163,   165,   167,   169,   171,
   173,   175,   177,   179,   181,   183,   185,   187,   190,   193,
   194,   196,   199,   202,   205,   208,   210,   212,   214,   217,
   219,   222,   224,   226,   228,   231,   233,   235,   237,   239,
   241,   243,   245,   248,   250,   253,   256,   258,   260,   262,
   264,   266,   269,   272,   275,   277,   279,   281,   283,   285,
   287,   288,   291,   293,   295,   297,   299,   301,   303,   305,
   307,   309
};

static const short yyrhs[] = {   102,
   103,     0,     0,   104,     0,   108,     0,   112,     0,     0,
    27,   130,   105,    93,   106,    94,     0,   106,   107,     0,
     1,     0,     0,   119,   117,   127,     0,     0,    73,   130,
   109,    93,   110,    94,     0,   110,   111,     0,     1,     0,
     0,   120,   117,   127,     0,     0,    75,   130,   113,    93,
   114,    94,     0,   114,   115,     0,     1,     0,     0,   124,
   126,   116,   127,     0,   120,     0,     0,   117,    95,   118,
     0,   118,     0,    86,     0,    71,     0,    76,     0,    77,
     0,    78,     0,    90,     0,    88,     0,    54,     0,     6,
     0,     4,     0,     1,     0,   121,    96,   123,    97,     0,
    96,   123,    97,     0,   121,    96,    72,    97,    92,     0,
    96,    72,    97,    92,     0,   121,   122,     0,   122,     0,
    28,     0,    87,     0,     3,     0,    74,     0,    79,     0,
    80,     0,    81,     0,    82,     0,    83,     0,     9,     0,
    10,     0,     8,     0,     7,     0,    13,     0,    14,     0,
    12,     0,    11,     0,    17,     0,    18,     0,    16,     0,
    15,     0,    21,     0,    22,     0,    20,     0,    19,     0,
    25,     0,    26,     0,    24,     0,    23,     0,     1,     0,
   130,     0,   125,     0,     1,     0,    98,   130,     0,    99,
   130,     0,     0,    29,     0,    30,   128,     0,    31,   128,
     0,    32,   130,     0,   100,   130,     0,    33,     0,    34,
     0,    35,     0,    36,   128,     0,    37,     0,    38,   130,
     0,    39,     0,    40,     0,    41,     0,    42,   128,     0,
    43,     0,    44,     0,    45,     0,    46,     0,    47,     0,
    48,     0,    49,     0,    50,   128,     0,    51,     0,    53,
   128,     0,    52,   128,     0,    56,     0,    57,     0,    58,
     0,    61,     0,    60,     0,    59,   128,     0,    62,   128,
     0,    63,   130,     0,    64,     0,    65,     0,    66,     0,
    67,     0,     1,     0,   129,     0,     0,   101,   130,     0,
    86,     0,    90,     0,    89,     0,    71,     0,    91,     0,
    55,     0,    84,     0,    85,     0,     5,     0,    92,     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
   264,   265,   268,   269,   270,   273,   292,   295,   296,   302,
   305,   411,   430,   433,   434,   440,   443,   495,   500,   506,
   507,   513,   516,   618,   620,   624,   626,   630,   632,   634,
   636,   638,   640,   642,   644,   646,   648,   650,   654,   659,
   666,   671,   678,   680,   684,   686,   688,   690,   692,   694,
   696,   698,   700,   704,   710,   716,   722,   728,   734,   740,
   746,   752,   758,   764,   770,   776,   782,   788,   794,   800,
   806,   812,   818,   824,   833,   835,   837,   845,   849,   859,
   863,   868,   873,   878,   883,   888,   893,   898,   903,   908,
   913,   918,   923,   928,   933,   938,   943,   948,   953,   958,
   963,   968,   973,   978,   983,   988,   993,   998,  1003,  1008,
  1013,  1018,  1023,  1028,  1033,  1038,  1043,  1048,  1053,  1061,
  1063,  1067,  1069,  1071,  1073,  1075,  1077,  1079,  1081,  1083,
  1085,  1088
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","ALT_TOK",
"APP_TOK","BACK_TOK","BORDER_TOK","BTN1_CLICK2_TOK","BTN1_CLICK_TOK","BTN1_DOWN_TOK",
"BTN1_UP_TOK","BTN2_CLICK2_TOK","BTN2_CLICK_TOK","BTN2_DOWN_TOK","BTN2_UP_TOK",
"BTN3_CLICK2_TOK","BTN3_CLICK_TOK","BTN3_DOWN_TOK","BTN3_UP_TOK","BTN4_CLICK2_TOK",
"BTN4_CLICK_TOK","BTN4_DOWN_TOK","BTN4_UP_TOK","BTN5_CLICK2_TOK","BTN5_CLICK_TOK",
"BTN5_DOWN_TOK","BTN5_UP_TOK","BUTTONS_TOK","CTRL_TOK","FBEEP_TOK","FCIRCLE_DOWN_TOK",
"FCIRCLE_UP_TOK","FEXEC_TOK","FFOCUS_COLOR_TOK","FFOCUS_KEY_TOK","FKILL_TOK",
"FLOWER_TOK","FMAXIMIZE_TOK","FMENU_TOK","FMINIMIZE_TOK","FMOVE_TOK","FNEXT_CMAP_TOK",
"FNEXT_KEY_TOK","FNOP_TOK","FNORMALIZE_TOK","FNORMANDRAISE_TOK","FPACK_ICONS_TOK",
"FPASS_KEYS_TOK","FPOST_WMENU_TOK","FPREV_CMAP_TOK","FPREV_KEY_TOK","FQUIT_MWM_TOK",
"FRAISE_LOWER_TOK","FRAISE_TOK","FRAME_TOK","FREE_FAMILY_TOK","FREFRESH_TOK",
"FREFRESH_WIN_TOK","FRESIZE_TOK","FRESTART_TOK","FRESTOREANDRAISE_TOK","FRESTORE_TOK",
"FSCREEN_TOK","FSEND_MSG_TOK","FSEPARATOR_TOK","FSET_BEHAVIOR_TOK","FTITLE_TOK",
"FWINDOWLIST_TOK","FDESK_TOK","FTOGGLE_PAGE_TOK","FGOTO_PAGE_TOK","ICON_TOK",
"KEY_TOK","KEYS_TOK","LOCK_TOK","MENU_TOK","MENUB_TOK","MINIMIZEB_TOK","MAXIMIZEB_TOK",
"MOD1_TOK","MOD2_TOK","MOD3_TOK","MOD4_TOK","MOD5_TOK","NEXT_TOK","PREV_TOK",
"ROOT_TOK","SHIFT_TOK","TITLE_TOK","TRANSIENT_TOK","WINDOW_TOK","WITHIN_TOK",
"STRING_TOK","'{'","'}'","'|'","'<'","'>'","'@'","'_'","'!'","'-'","res_file",
"binding","button_bindings","@1","button_list","button_binding","key_bindings",
"@2","key_list","key_binding","menu_bindings","@3","item_list","item_binding",
"accelerator","context","object","button","key","modifier_list","modifier_name",
"button_event_name","label","bitmap_file","mnemonic","function","optional_arg",
"arg","string", NULL
};
#endif

static const short yyr1[] = {     0,
   102,   102,   103,   103,   103,   105,   104,   106,   106,   106,
   107,   109,   108,   110,   110,   110,   111,   113,   112,   114,
   114,   114,   115,   116,   116,   117,   117,   118,   118,   118,
   118,   118,   118,   118,   118,   118,   118,   118,   119,   119,
   120,   120,   121,   121,   122,   122,   122,   122,   122,   122,
   122,   122,   122,   123,   123,   123,   123,   123,   123,   123,
   123,   123,   123,   123,   123,   123,   123,   123,   123,   123,
   123,   123,   123,   123,   124,   124,   124,   125,   126,   126,
   127,   127,   127,   127,   127,   127,   127,   127,   127,   127,
   127,   127,   127,   127,   127,   127,   127,   127,   127,   127,
   127,   127,   127,   127,   127,   127,   127,   127,   127,   127,
   127,   127,   127,   127,   127,   127,   127,   127,   127,   128,
   128,   129,   129,   129,   129,   129,   129,   129,   129,   129,
   129,   130
};

static const short yyr2[] = {     0,
     2,     0,     1,     1,     1,     0,     6,     2,     1,     0,
     3,     0,     6,     2,     1,     0,     3,     0,     6,     2,
     1,     0,     4,     1,     0,     3,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     4,     3,
     5,     4,     2,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     2,     2,     0,
     1,     2,     2,     2,     2,     1,     1,     1,     2,     1,
     2,     1,     1,     1,     2,     1,     1,     1,     1,     1,
     1,     1,     2,     1,     2,     2,     1,     1,     1,     1,
     1,     2,     2,     2,     1,     1,     1,     1,     1,     1,
     0,     2,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1
};

static const short yydefact[] = {     2,
     0,     0,     0,     0,     1,     3,     4,     5,   132,     6,
    12,    18,     0,     0,     0,     0,     0,     0,     9,     0,
    15,     0,    21,     0,    47,    45,    48,    49,    50,    51,
    52,    53,    46,     7,     0,     8,     0,     0,    44,    13,
     0,    14,     0,     0,    77,    19,     0,    20,    80,    76,
    75,    74,    57,    56,    54,    55,    61,    60,    58,    59,
    65,    64,    62,    63,    69,    68,    66,    67,    73,    72,
    70,    71,     0,    38,    37,    36,    35,    29,    30,    31,
    32,    28,    34,    33,     0,    27,     0,    43,     0,     0,
     0,    78,     0,    25,    40,   119,    81,   121,   121,     0,
    86,    87,    88,   121,    90,     0,    92,    93,    94,   121,
    96,    97,    98,    99,   100,   101,   102,   121,   104,   121,
   121,   107,   108,   109,   121,   111,   110,   121,     0,   115,
   116,   117,   118,     0,     0,    11,     0,     0,    17,     0,
    79,     0,    24,   131,   128,   126,   129,   130,   123,   125,
   124,   127,     0,    82,   120,    83,    84,    89,    91,    95,
   103,   106,   105,   112,   113,   114,    26,    85,    39,    42,
     0,    23,   122,    41,     0,     0
};

static const short yydefgoto[] = {     1,
     5,     6,    13,    20,    36,     7,    14,    22,    42,     8,
    15,    24,    48,   142,    85,    86,    37,    43,    44,    39,
    73,    49,    50,    94,   136,   154,   155,    10
};

static const short yypact[] = {-32768,
    11,   -87,   -87,   -87,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   -86,   -85,   -83,   117,   200,     1,-32768,   174,
-32768,   224,-32768,     2,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   359,-32768,    31,   234,-32768,-32768,
   -59,-32768,    31,   257,-32768,-32768,   -87,-32768,   -80,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   -81,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,    16,-32768,   359,-32768,   -77,    16,
   -58,-32768,   -87,   269,-32768,-32768,-32768,   135,   135,   -87,
-32768,-32768,-32768,   135,-32768,   -87,-32768,-32768,-32768,   135,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,   135,-32768,   135,
   135,-32768,-32768,-32768,   135,-32768,-32768,   135,   -87,-32768,
-32768,-32768,-32768,    31,   -87,-32768,   -75,   -68,-32768,   -70,
-32768,   122,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   -87,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
   -64,-32768,-32768,-32768,    29,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   -12,  -100,-32768,   -55,    20,   -32,
   -51,-32768,-32768,-32768,   -72,   -95,-32768,    -3
};


#define	YYLAST		385


static const short yytable[] = {    11,
    12,    23,    45,   156,     9,    88,    16,    17,   158,    18,
   175,    88,    89,   140,   160,    95,    96,   139,    93,   138,
    51,   169,   161,   170,   162,   163,   171,   174,   176,   164,
    90,    74,   165,   167,    75,   137,    76,     2,   143,    38,
     0,     0,     0,    92,    97,    98,    99,   100,   101,   102,
   103,   104,   105,   106,   107,   108,   109,   110,   111,   112,
   113,   114,   115,   116,   117,   118,   119,   120,   121,   172,
     0,   122,   123,   124,   125,   126,   127,   128,   129,   130,
   131,   132,   133,     3,    77,     4,     0,     0,     0,   141,
     0,     0,   -22,     9,   -22,    46,   157,     0,   -22,    47,
     0,    78,   159,     0,     0,     0,    79,    80,    81,     0,
   134,     0,     0,     0,     0,   135,    82,    19,    83,   -10,
    84,     0,    96,     0,     0,   166,     0,     0,     0,     0,
     0,   168,     0,     0,     0,     0,     0,     0,     0,   144,
     0,     0,     0,     0,   -10,     0,     0,     0,     0,   173,
    97,    98,    99,   100,   101,   102,   103,   104,   105,   106,
   107,   108,   109,   110,   111,   112,   113,   114,   115,   116,
   117,   118,   119,   120,   121,     0,    25,   122,   123,   124,
   125,   126,   127,   128,   129,   130,   131,   132,   133,   145,
   -10,     0,     0,     0,     0,   -10,   -10,   -10,   -10,   -10,
    21,    26,   -16,   -10,     0,   146,     0,     0,     0,     0,
   -10,     0,   -10,     0,     0,     0,     0,     0,   147,   148,
   149,   135,     0,   150,   151,   152,    25,   -16,     0,     0,
     0,     0,     0,     0,     0,   153,    25,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    27,     0,     0,
     0,    26,    28,    29,    30,    31,    32,     0,     0,    25,
    33,    26,     0,     0,     0,     0,     0,    34,     0,    35,
     0,    25,     0,   -16,     0,     0,     0,     0,   -16,   -16,
   -16,   -16,   -16,     0,    26,     0,   -16,     0,     0,     0,
     0,     0,     0,   -16,     0,   -16,    26,    27,     0,     0,
     0,     0,    28,    29,    30,    31,    32,    27,     0,     0,
    33,     0,    28,    29,    30,    31,    32,    40,     0,    41,
    33,     0,     0,     0,     0,     0,     0,     0,     0,    87,
    27,     0,     0,     0,     0,    28,    29,    30,    31,    32,
     0,     0,    27,    33,     0,     0,     0,    28,    29,    30,
    31,    32,    91,     0,     0,    33,     0,     0,     0,    52,
     0,     0,     0,     0,    41,    53,    54,    55,    56,    57,
    58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
    68,    69,    70,    71,    72
};

static const short yycheck[] = {     3,
     4,     1,     1,    99,    92,    38,    93,    93,   104,    93,
     0,    44,    72,    72,   110,    97,     1,    90,    99,    97,
    24,    97,   118,    92,   120,   121,    97,    92,     0,   125,
    43,     1,   128,   134,     4,    87,     6,    27,    94,    20,
    -1,    -1,    -1,    47,    29,    30,    31,    32,    33,    34,
    35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    47,    48,    49,    50,    51,    52,    53,   142,
    -1,    56,    57,    58,    59,    60,    61,    62,    63,    64,
    65,    66,    67,    73,    54,    75,    -1,    -1,    -1,    93,
    -1,    -1,    92,    92,    94,    94,   100,    -1,    98,    98,
    -1,    71,   106,    -1,    -1,    -1,    76,    77,    78,    -1,
    95,    -1,    -1,    -1,    -1,   100,    86,     1,    88,     3,
    90,    -1,     1,    -1,    -1,   129,    -1,    -1,    -1,    -1,
    -1,   135,    -1,    -1,    -1,    -1,    -1,    -1,    -1,     5,
    -1,    -1,    -1,    -1,    28,    -1,    -1,    -1,    -1,   153,
    29,    30,    31,    32,    33,    34,    35,    36,    37,    38,
    39,    40,    41,    42,    43,    44,    45,    46,    47,    48,
    49,    50,    51,    52,    53,    -1,     3,    56,    57,    58,
    59,    60,    61,    62,    63,    64,    65,    66,    67,    55,
    74,    -1,    -1,    -1,    -1,    79,    80,    81,    82,    83,
     1,    28,     3,    87,    -1,    71,    -1,    -1,    -1,    -1,
    94,    -1,    96,    -1,    -1,    -1,    -1,    -1,    84,    85,
    86,   100,    -1,    89,    90,    91,     3,    28,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,   101,     3,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    74,    -1,    -1,
    -1,    28,    79,    80,    81,    82,    83,    -1,    -1,     3,
    87,    28,    -1,    -1,    -1,    -1,    -1,    94,    -1,    96,
    -1,     3,    -1,    74,    -1,    -1,    -1,    -1,    79,    80,
    81,    82,    83,    -1,    28,    -1,    87,    -1,    -1,    -1,
    -1,    -1,    -1,    94,    -1,    96,    28,    74,    -1,    -1,
    -1,    -1,    79,    80,    81,    82,    83,    74,    -1,    -1,
    87,    -1,    79,    80,    81,    82,    83,    94,    -1,    96,
    87,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    96,
    74,    -1,    -1,    -1,    -1,    79,    80,    81,    82,    83,
    -1,    -1,    74,    87,    -1,    -1,    -1,    79,    80,    81,
    82,    83,    96,    -1,    -1,    87,    -1,    -1,    -1,     1,
    -1,    -1,    -1,    -1,    96,     7,     8,     9,    10,    11,
    12,    13,    14,    15,    16,    17,    18,    19,    20,    21,
    22,    23,    24,    25,    26
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/share/bison.simple"
/* This file comes from bison-1.28.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

#ifndef YYPARSE_RETURN_TYPE
#define YYPARSE_RETURN_TYPE int
#endif


#ifndef YYSTACK_USE_ALLOCA
#ifdef alloca
#define YYSTACK_USE_ALLOCA
#else /* alloca not defined */
#ifdef __GNUC__
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
#define YYSTACK_USE_ALLOCA
#include <alloca.h>
#else /* not sparc */
/* We think this test detects Watcom and Microsoft C.  */
/* This used to test MSDOS, but that is a bad idea
   since that symbol is in the user namespace.  */
#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
#if 0 /* No need for malloc.h, which pollutes the namespace;
	 instead, just don't use alloca.  */
#include <malloc.h>
#endif
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
/* I don't know what this was needed for, but it pollutes the namespace.
   So I turned it off.   rms, 2 May 1997.  */
/* #include <malloc.h>  */
 #pragma alloca
#define YYSTACK_USE_ALLOCA
#else /* not MSDOS, or __TURBOC__, or _AIX */
#if 0
#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
		 and on HPUX 10.  Eventually we can turn this on.  */
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#endif /* __hpux */
#endif
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc */
#endif /* not GNU C */
#endif /* alloca not defined */
#endif /* YYSTACK_USE_ALLOCA not defined */

#ifdef YYSTACK_USE_ALLOCA
#define YYSTACK_ALLOC alloca
#else
#define YYSTACK_ALLOC malloc
#endif

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto yyacceptlab
#define YYABORT 	goto yyabortlab
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Define __yy_memcpy.  Note that the size argument
   should be passed with type unsigned int, because that is what the non-GCC
   definitions require.  With GCC, __builtin_memcpy takes an arg
   of type size_t, but it can handle unsigned int.  */

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     unsigned int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, unsigned int count)
{
  register char *t = to;
  register char *f = from;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 222 "/usr/share/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
#ifdef YYPARSE_PARAM
YYPARSE_RETURN_TYPE
yyparse (void *);
#else
YYPARSE_RETURN_TYPE
yyparse (void);
#endif
#endif

YYPARSE_RETURN_TYPE
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;
#ifndef YYSTACK_USE_ALLOCA
  int yyfree_stacks = 0;
#endif

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
#ifndef YYSTACK_USE_ALLOCA
	  if (yyfree_stacks)
	    {
	      free (yyss);
	      free (yyvs);
#ifdef YYLSP_NEEDED
	      free (yyls);
#endif
	    }
#endif	    
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
#ifndef YYSTACK_USE_ALLOCA
      yyfree_stacks = 1;
#endif
      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1,
		   size * (unsigned int) sizeof (*yyssp));
      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
		   size * (unsigned int) sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1,
		   size * (unsigned int) sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 6:
#line 274 "mwmparse.y"
{
			    if (!skip_test) {
				if (button_bind_found == False) {
				    if (strcmp(yyvsp[0].string, pscr->button_bindings) == 0)
					button_bind_match = True;
				    else
					button_bind_match = False;
				}
				else if (strcmp(yyvsp[0].string, pscr->button_bindings) == 0) {
				    yyerror("Ignoring duplicate button bindings");
				    button_bind_match = False;
				}
				else
				    button_bind_match = False;
			    }
			    else
				button_bind_match = True;
			;
    break;}
case 9:
#line 297 "mwmparse.y"
{
			    yyerror("Invalid button binding");
			    yyerrok;
			    yyclearin;
			;
    break;}
case 11:
#line 306 "mwmparse.y"
{
			    int contexts = yyvsp[-1].number;
			    int mods, func;
			    MenuItem *mi = NULL;
			    MouseButton *temp = NULL, *ptr;

			    if (!button_bind_match)
				break;

			    mods = yyvsp[-2].button.modifiers;
			    if ((contexts & C_WINDOW) &&
				(((mods == 0) || mods == AnyModifier))) {
			        pscr->buttons2grab &= ~(yyvsp[-2].button.button);
			    }
			
			    func = yyvsp[0].function.func;

			    if ((func == F_EXEC) ||
				(func == F_RESTART) ||
			        (func == F_CIRCULATE_UP) ||
				(func == F_CIRCULATE_DOWN) ||
			        (func == F_WARP)) {
			        mi = (MenuItem *)XtMalloc(sizeof(MenuItem));
			
			        mi->next = (MenuItem *) NULL;
			        mi->prev = (MenuItem *) NULL;
			        if ((func == F_EXEC) || (func == F_RESTART)) {
			            mi->item = XtNewString("DOIT");
			            mi->action = yyvsp[0].function.arg;
			        }
			        else {
			            mi->item = XtNewString("DONT DOIT");
			            mi->action = yyvsp[0].function.arg;
			        }
				mi->item2 = "";
			        mi->state = 0;
			        mi->func = func;
			        mi->strlen = strlen(mi->item);
				mi->strlen2 = 0;
			        mi->val1 = 0;
			        mi->val2 = 0;
			        mi->val1_unit = 1;
			        mi->val2_unit = 1;
			    }

			    temp = pscr->buttons;
			    pscr->buttons =
				(MouseButton *)XtMalloc(sizeof(MouseButton));
			    pscr->buttons->func = func;
			    if (func == F_POPUP)
				pscr->buttons->menu = (MenuRoot *)yyvsp[0].function.arg;
			    else if (func == F_W_POPUP)
				pscr->buttons->menu = (MenuRoot *)DEFAULT_WIN_MENU_NAME;
			    else
				pscr->buttons->menu = NULL;
			    pscr->buttons->item = mi;
			    pscr->buttons->button =
				ffs(yyvsp[-2].button.button) - 8;
			    pscr->buttons->context = contexts;
			    pscr->buttons->modifier = mods;
			    pscr->buttons->mask = yyvsp[-2].button.event;
			    pscr->buttons->count = yyvsp[-2].button.count;
			    pscr->buttons->val1 = 0;
			    pscr->buttons->val2 = 0;
			    pscr->buttons->val1_unit = pscr->d_width;
			    pscr->buttons->val2_unit = pscr->d_height;

			    /* if duplicate, skip second */
			    for (ptr = temp;
				 ptr != NULL;
				 ptr = ptr->next) {
                              /* If everything is the same except for having
                                  func's F_POPUP and F_WINDOWLIST, the
                                  second is a duplicate */
                               if (((ptr->func == F_POPUP || 
                                     ptr->func == F_WINDOWLIST) && 
                                    (pscr->buttons->func == F_POPUP || 
                                     pscr->buttons->func == F_WINDOWLIST)) &&
				    ptr->modifier == pscr->buttons->modifier &&
				    ptr->mask == pscr->buttons->mask &&
                                   ptr->count == pscr->buttons->count &&
                                   ptr->context == pscr->buttons->context &&
                                   ptr->button  == pscr->buttons->button) {

                                   XtFree((char *)pscr->buttons);
                                   pscr->buttons = temp;
                                   break;
                               }

                               if (ptr->func == pscr->buttons->func &&
                                   ptr->modifier == pscr->buttons->modifier &&
                                   ptr->mask == pscr->buttons->mask &&
                                   ptr->count == pscr->buttons->count &&
                                   ptr->button == pscr->buttons->button) {
				    ptr->context |= pscr->buttons->context;
				    XtFree((char *)pscr->buttons);
				    pscr->buttons = temp;
				    break;
				}
			    }
			    if (ptr == NULL)
				pscr->buttons->next = temp;
			;
    break;}
case 12:
#line 412 "mwmparse.y"
{
			    if (!skip_test) {
				if (key_bind_found == False) {
				    if (strcmp(yyvsp[0].string, pscr->key_bindings) == 0)
					key_bind_match = True;
				    else
					key_bind_match = False;
				}
				else if (strcmp(yyvsp[0].string, pscr->key_bindings) == 0) {
				    yyerror("Ignoring duplicate key bindings");
					key_bind_match = False;
				}
				else
				    key_bind_match = False;
			    }
			    else
				key_bind_match = True;
			;
    break;}
case 15:
#line 435 "mwmparse.y"
{
			    yyerror("Invalid key binding");
			    yyerrok;
			    yyclearin;
			;
    break;}
case 17:
#line 444 "mwmparse.y"
{
			    FuncKey        *tmp = NULL;
			    KeySym          keysym;
			    KeyCode         keycode;
			    int             i, min, max;
			    int func, contexts;
			    char *ptr;

			    if (!key_bind_match)
				break;

			    ptr = yyvsp[0].function.arg;
			    func = yyvsp[0].function.func;
			    contexts = yyvsp[-1].number;

			    /*
			     * Don't let a 0 keycode go through, since that
			     * means AnyKey to the XGrabKey call in GrabKeys().
			     */
			    keysym = yyvsp[-2].hotkey.key;
			    if (keysym == 0)
				break;
			    if ((keycode = XKeysymToKeycode(dpy, yyvsp[-2].hotkey.key)) == 0)
				break;

			    XDisplayKeycodes(dpy, &min, &max);
			    for (i = min; i <= max; i++) {
				if (XKeycodeToKeysym(dpy, i, 0) == keysym) {
				    tmp = (FuncKey *)XtMalloc(sizeof(FuncKey));
				    tmp->next = pscr->keys;
				    pscr->keys = tmp;

				    tmp->name = "HOTKEY";
				    tmp->keycode = i;
				    tmp->cont = contexts;
				    tmp->mods = yyvsp[-2].hotkey.modifiers;
				    tmp->func = func;
				    if (func == F_W_POPUP)
					tmp->action = DEFAULT_WIN_MENU_NAME;
				    else
					tmp->action = ptr;
				    tmp->val1 = 0;
				    tmp->val2 = 0;
				    tmp->val1_unit = pscr->d_width;
				    tmp->val2_unit = pscr->d_height;
				    tmp->menu = NULL;
				}
			    }
			;
    break;}
case 18:
#line 496 "mwmparse.y"
{
			    num_items = 0;
			    cur_menu = MENU_Create(yyvsp[0].string);
			;
    break;}
case 19:
#line 501 "mwmparse.y"
{
			    MENU_Add(pscr, cur_menu);
			;
    break;}
case 21:
#line 508 "mwmparse.y"
{
			    yyerror("Invalid menu item");
			    yyerrok;
			    yyclearin;
			;
    break;}
case 23:
#line 517 "mwmparse.y"
{
			    MenuItem *tmp;
			    int width, width2;
			    FuncKey        *tmpk = NULL;
			    KeySym          keysym;
			    KeyCode         keycode;
			    int             i, min, max;
			    int func, contexts;
			    char *ptr, *acc;

			    tmp = (MenuItem *)XtCalloc(1, sizeof(MenuItem));
			    if (cur_menu->first == NULL) {
				cur_menu->first = tmp;
				tmp->prev = NULL;
				tmp->next = NULL;
			    }
			    else {
				cur_menu->last->next = tmp;
				tmp->prev = cur_menu->last;
			    }
			    cur_menu->last = tmp;
			    cur_menu->items++;

			    if (yyvsp[0].function.func == F_NOP)
				tmp->item = "";
			    else
				tmp->item = yyvsp[-3].label.string;
			    tmp->item2 = "";
			    MENU_FindHotKey(tmp, yyvsp[-2].key);
			    if (strcmp(tmp->item, "no-label") == 0)
				tmp->strlen = 0;
			    else
				tmp->strlen = strlen(tmp->item);

			    tmp->action = yyvsp[0].function.arg;
			    tmp->next = NULL;
			    tmp->state = 0;
			    tmp->func = yyvsp[0].function.func;
			    tmp->val1 = 0;
			    tmp->val2 = 0;
			    tmp->val1_unit = pscr->d_width;
			    tmp->val2_unit = pscr->d_height;
			
			    width = XTextWidth(pscr->components[MWM_MENU].font,
					       tmp->item, tmp->strlen);
			    if (tmp->func == F_POPUP)
			        width += 15;
			    if (width <= 0)
			        width = 1;
			    if (width > cur_menu->width)
			        cur_menu->width = width;

			    ptr = yyvsp[0].function.arg;
			    func = yyvsp[0].function.func;
			    contexts = C_WINDOW|C_ICON|C_FRAME|C_TITLE|C_LALL|C_RALL;

			    /*
			     * Don't let a 0 keycode go through, since that
			     * means AnyKey to the XGrabKey call in GrabKeys().
			     */
			    keysym = yyvsp[-1].hotkey.key;
			    if (keysym == 0)
				break;
			    if ((keycode = XKeysymToKeycode(dpy, yyvsp[-1].hotkey.key)) == 0)
				break;

			    acc = MENU_AcceleratorString(pscr,
							 yyvsp[-1].hotkey.key,
							 yyvsp[-1].hotkey.modifiers);
			    if (strlen(acc)) {
				tmp->item2 = XtNewString(acc);
				tmp->strlen2 = strlen(acc);
				width2 = XTextWidth(pscr->components[MWM_MENU].font,
						    tmp->item2, tmp->strlen2);
				if (width2 > cur_menu->width2)
				    cur_menu->width2 = width2;
			    }

			    XDisplayKeycodes(dpy, &min, &max);
			    for (i = min; i <= max; i++) {
				if (XKeycodeToKeysym(dpy, i, 0) == keysym) {
				    tmpk = (FuncKey *)XtMalloc(sizeof(FuncKey));
				    tmpk->next = pscr->keys;
				    pscr->keys = tmpk;

				    tmpk->name = "HOTKEY";
				    tmpk->keycode = i;
				    tmpk->cont = contexts;
				    tmpk->mods = yyvsp[-1].hotkey.modifiers;
				    tmpk->func = func;
				    tmpk->action = ptr;
				    tmpk->val1 = 0;
				    tmpk->val2 = 0;
				    tmpk->val1_unit = pscr->d_width;
				    tmpk->val2_unit = pscr->d_height;
				    tmpk->menu = NULL;
				}
			    }
			;
    break;}
case 24:
#line 619 "mwmparse.y"
{ yyval.hotkey = yyvsp[0].hotkey; ;
    break;}
case 25:
#line 621 "mwmparse.y"
{ yyval.hotkey.modifiers = 0; yyval.hotkey.key = 0; ;
    break;}
case 26:
#line 625 "mwmparse.y"
{ yyval.number = yyvsp[-2].number | yyvsp[0].number; ;
    break;}
case 27:
#line 627 "mwmparse.y"
{ yyval.number = yyvsp[0].number; ;
    break;}
case 28:
#line 631 "mwmparse.y"
{ yyval.number = C_ROOT; ;
    break;}
case 29:
#line 633 "mwmparse.y"
{ yyval.number = C_ICON; ;
    break;}
case 30:
#line 635 "mwmparse.y"
{ yyval.number = C_MENUB; ;
    break;}
case 31:
#line 637 "mwmparse.y"
{ yyval.number = C_MINIMIZEB; ;
    break;}
case 32:
#line 639 "mwmparse.y"
{ yyval.number = C_MAXIMIZEB; ;
    break;}
case 33:
#line 641 "mwmparse.y"
{ yyval.number = C_WINDOW|C_FRAME|C_TITLE|C_LALL|C_RALL|C_ICON; ;
    break;}
case 34:
#line 643 "mwmparse.y"
{ yyval.number = C_TITLE; ;
    break;}
case 35:
#line 645 "mwmparse.y"
{ yyval.number = C_FRAME|C_TITLE|C_LALL|C_RALL; ;
    break;}
case 36:
#line 647 "mwmparse.y"
{ yyval.number = C_FRAME; ;
    break;}
case 37:
#line 649 "mwmparse.y"
{ yyval.number = C_WINDOW; ;
    break;}
case 38:
#line 651 "mwmparse.y"
{ yyerror("Invalid context"); yyval.number = 0; ;
    break;}
case 39:
#line 655 "mwmparse.y"
{
			    yyval.button = yyvsp[-1].button;
			    yyval.button.modifiers = yyvsp[-3].modifiers;
			;
    break;}
case 40:
#line 660 "mwmparse.y"
{
			    yyval.button = yyvsp[-1].button;
			    yyval.button.modifiers = 0;
			;
    break;}
case 41:
#line 667 "mwmparse.y"
{
			    yyval.hotkey.modifiers = yyvsp[-4].modifiers;
			    yyval.hotkey.key = XStringToKeysym(yyvsp[0].string);
			;
    break;}
case 42:
#line 672 "mwmparse.y"
{
			    yyval.hotkey.modifiers = 0;
			    yyval.hotkey.key = XStringToKeysym(yyvsp[0].string);
			;
    break;}
case 43:
#line 679 "mwmparse.y"
{ yyval.modifiers = yyvsp[-1].modifiers | yyvsp[0].modifiers; ;
    break;}
case 44:
#line 681 "mwmparse.y"
{ yyval.modifiers = yyvsp[0].modifiers; ;
    break;}
case 45:
#line 685 "mwmparse.y"
{ yyval.modifiers = ControlMask; ;
    break;}
case 46:
#line 687 "mwmparse.y"
{ yyval.modifiers = ShiftMask; ;
    break;}
case 47:
#line 689 "mwmparse.y"
{ yyval.modifiers = pscr->alt_mask; ;
    break;}
case 48:
#line 691 "mwmparse.y"
{ yyval.modifiers = LockMask; ;
    break;}
case 49:
#line 693 "mwmparse.y"
{ yyval.modifiers = Mod1Mask; ;
    break;}
case 50:
#line 695 "mwmparse.y"
{ yyval.modifiers = Mod2Mask; ;
    break;}
case 51:
#line 697 "mwmparse.y"
{ yyval.modifiers = Mod3Mask; ;
    break;}
case 52:
#line 699 "mwmparse.y"
{ yyval.modifiers = Mod4Mask; ;
    break;}
case 53:
#line 701 "mwmparse.y"
{ yyval.modifiers = Mod5Mask; ;
    break;}
case 54:
#line 705 "mwmparse.y"
{
			    yyval.button.button = Button1Mask;
			    yyval.button.event = ButtonPressMask;
			    yyval.button.count = 0;
			;
    break;}
case 55:
#line 711 "mwmparse.y"
{
			    yyval.button.button = Button1Mask;
			    yyval.button.event = ButtonReleaseMask;
			    yyval.button.count = 0;
			;
    break;}
case 56:
#line 717 "mwmparse.y"
{
			    yyval.button.button = Button1Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 1;
			;
    break;}
case 57:
#line 723 "mwmparse.y"
{
			    yyval.button.button = Button1Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 2;
			;
    break;}
case 58:
#line 729 "mwmparse.y"
{
			    yyval.button.button = Button2Mask;
			    yyval.button.event = ButtonPressMask;
			    yyval.button.count = 0;
			;
    break;}
case 59:
#line 735 "mwmparse.y"
{
			    yyval.button.button = Button2Mask;
			    yyval.button.event = ButtonReleaseMask;
			    yyval.button.count = 0;
			;
    break;}
case 60:
#line 741 "mwmparse.y"
{
			    yyval.button.button = Button2Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 1;
			;
    break;}
case 61:
#line 747 "mwmparse.y"
{
			    yyval.button.button = Button2Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 2;
			;
    break;}
case 62:
#line 753 "mwmparse.y"
{
			    yyval.button.button = Button3Mask;
			    yyval.button.event = ButtonPressMask;
			    yyval.button.count = 0;
			;
    break;}
case 63:
#line 759 "mwmparse.y"
{
			    yyval.button.button = Button3Mask;
			    yyval.button.event = ButtonReleaseMask;
			    yyval.button.count = 0;
			;
    break;}
case 64:
#line 765 "mwmparse.y"
{
			    yyval.button.button = Button3Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 1;
			;
    break;}
case 65:
#line 771 "mwmparse.y"
{
			    yyval.button.button = Button3Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 2;
			;
    break;}
case 66:
#line 777 "mwmparse.y"
{
			    yyval.button.button = Button4Mask;
			    yyval.button.event = ButtonPressMask;
			    yyval.button.count = 0;
			;
    break;}
case 67:
#line 783 "mwmparse.y"
{
			    yyval.button.button = Button4Mask;
			    yyval.button.event = ButtonReleaseMask;
			    yyval.button.count = 0;
			;
    break;}
case 68:
#line 789 "mwmparse.y"
{
			    yyval.button.button = Button4Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 1;
			;
    break;}
case 69:
#line 795 "mwmparse.y"
{
			    yyval.button.button = Button4Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 2;
			;
    break;}
case 70:
#line 801 "mwmparse.y"
{
			    yyval.button.button = Button5Mask;
			    yyval.button.event = ButtonPressMask;
			    yyval.button.count = 0;
			;
    break;}
case 71:
#line 807 "mwmparse.y"
{
			    yyval.button.button = Button5Mask;
			    yyval.button.event = ButtonReleaseMask;
			    yyval.button.count = 0;
			;
    break;}
case 72:
#line 813 "mwmparse.y"
{
			    yyval.button.button = Button5Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 1;
			;
    break;}
case 73:
#line 819 "mwmparse.y"
{
			    yyval.button.button = Button5Mask;
			    yyval.button.event = ButtonPressMask|ButtonReleaseMask;
			    yyval.button.count = 2;
			;
    break;}
case 74:
#line 825 "mwmparse.y"
{
			    yyerror("Invalid button event name");
			    yyval.button.button = 0;
			    yyval.button.event = 0;
			    yyval.button.count = 0;
			;
    break;}
case 75:
#line 834 "mwmparse.y"
{ yyval.label.type = IS_STRING; yyval.label.string = yyvsp[0].string; ;
    break;}
case 76:
#line 836 "mwmparse.y"
{ yyval.label.type = IS_BITMAP; yyval.label.string = yyvsp[0].string; ;
    break;}
case 77:
#line 838 "mwmparse.y"
{
			    yyerror("Invalid label");
			    yyval.label.type = IS_STRING;
			    yyval.label.string = "";
			;
    break;}
case 78:
#line 846 "mwmparse.y"
{ yyval.string = yyvsp[0].string; ;
    break;}
case 79:
#line 850 "mwmparse.y"
{
			    if (strlen(yyvsp[0].string) != 1) {
				yyerror("Invalid mnemonic specification");
				yyval.key = 0;
			    }
			    else {
				yyval.key = XStringToKeysym(yyvsp[0].string);
			    }
			;
    break;}
case 80:
#line 860 "mwmparse.y"
{ yyval.key = 0; ;
    break;}
case 81:
#line 864 "mwmparse.y"
{
			    yyval.function.func = F_BEEP;
			    yyval.function.arg = "";
			;
    break;}
case 82:
#line 869 "mwmparse.y"
{
			    yyval.function.func = F_CIRCULATE_DOWN;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 83:
#line 874 "mwmparse.y"
{
			    yyval.function.func = F_CIRCULATE_UP;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 84:
#line 879 "mwmparse.y"
{
			    yyval.function.func = F_EXEC;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 85:
#line 884 "mwmparse.y"
{
			    yyval.function.func = F_EXEC;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 86:
#line 889 "mwmparse.y"
{
			    yyval.function.func = F_FOCUS_COLOR;
			    yyval.function.arg = "";
			;
    break;}
case 87:
#line 894 "mwmparse.y"
{
			    yyval.function.func = F_FOCUS_KEY;
			    yyval.function.arg = "";
			;
    break;}
case 88:
#line 899 "mwmparse.y"
{
			    yyval.function.func = F_CLOSE;
			    yyval.function.arg = "";
			;
    break;}
case 89:
#line 904 "mwmparse.y"
{
			    yyval.function.func = F_LOWER;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 90:
#line 909 "mwmparse.y"
{
			    yyval.function.func = F_MAXIMIZE;
			    yyval.function.arg = "";
			;
    break;}
case 91:
#line 914 "mwmparse.y"
{
			    yyval.function.func = F_POPUP;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 92:
#line 919 "mwmparse.y"
{
			    yyval.function.func = F_ICONIFY;
			    yyval.function.arg = "";
			;
    break;}
case 93:
#line 924 "mwmparse.y"
{
			    yyval.function.func = F_MOVE;
			    yyval.function.arg = "";
			;
    break;}
case 94:
#line 929 "mwmparse.y"
{
			    yyval.function.func = F_NEXT_CMAP;
			    yyval.function.arg = "";
			;
    break;}
case 95:
#line 934 "mwmparse.y"
{
			    yyval.function.func = F_NEXT_KEY;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 96:
#line 939 "mwmparse.y"
{
			    yyval.function.func = F_NOP;
			    yyval.function.arg = "";
			;
    break;}
case 97:
#line 944 "mwmparse.y"
{
			    yyval.function.func = F_NORMALIZE;
			    yyval.function.arg = "";
			;
    break;}
case 98:
#line 949 "mwmparse.y"
{
			    yyval.function.func = F_NORM_AND_RAISE;
			    yyval.function.arg = "";
			;
    break;}
case 99:
#line 954 "mwmparse.y"
{
			    yyval.function.func = F_PACK_ICONS;
			    yyval.function.arg = "";
			;
    break;}
case 100:
#line 959 "mwmparse.y"
{
			    yyval.function.func = F_PASS_KEYS;
			    yyval.function.arg = "";
			;
    break;}
case 101:
#line 964 "mwmparse.y"
{
			    yyval.function.func = F_W_POPUP;
			    yyval.function.arg = "";
			;
    break;}
case 102:
#line 969 "mwmparse.y"
{
			    yyval.function.func = F_PREV_CMAP;
			    yyval.function.arg = "";
			;
    break;}
case 103:
#line 974 "mwmparse.y"
{
			    yyval.function.func = F_PREV_KEY;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 104:
#line 979 "mwmparse.y"
{
			    yyval.function.func = F_QUIT;
			    yyval.function.arg = "";
			;
    break;}
case 105:
#line 984 "mwmparse.y"
{
			    yyval.function.func = F_RAISE;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 106:
#line 989 "mwmparse.y"
{
			    yyval.function.func = F_RAISELOWER;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 107:
#line 994 "mwmparse.y"
{
			    yyval.function.func = F_REFRESH;
			    yyval.function.arg = "";
			;
    break;}
case 108:
#line 999 "mwmparse.y"
{
			    yyval.function.func = F_REFRESH_WIN;
			    yyval.function.arg = "";
			;
    break;}
case 109:
#line 1004 "mwmparse.y"
{
			    yyval.function.func = F_RESIZE;
			    yyval.function.arg = "";
			;
    break;}
case 110:
#line 1009 "mwmparse.y"
{
			    yyval.function.func = F_ICONIFY;
			    yyval.function.arg = "";
			;
    break;}
case 111:
#line 1014 "mwmparse.y"
{
			    yyval.function.func = F_RESTORE_AND_RAISE;
			    yyval.function.arg = "";
			;
    break;}
case 112:
#line 1019 "mwmparse.y"
{
			    yyval.function.func = F_RESTART;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 113:
#line 1024 "mwmparse.y"
{
			    yyval.function.func = F_SCREEN;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 114:
#line 1029 "mwmparse.y"
{
			    yyval.function.func = F_SEND_MSG;
			    yyval.function.arg = yyvsp[0].string;
			;
    break;}
case 115:
#line 1034 "mwmparse.y"
{
			    yyval.function.func = F_NOP;
			    yyval.function.arg = "";
			;
    break;}
case 116:
#line 1039 "mwmparse.y"
{
			    yyval.function.func = F_SET_BEHAVIOR;
			    yyval.function.arg = "";
			;
    break;}
case 117:
#line 1044 "mwmparse.y"
{
			    yyval.function.func = F_TITLE;
			    yyval.function.arg = "";
			;
    break;}
case 118:
#line 1049 "mwmparse.y"
{
			    yyval.function.func = F_WINDOWLIST;
			    yyval.function.arg = "";
			;
    break;}
case 119:
#line 1054 "mwmparse.y"
{
			    yyerror("Invalid mnemonic, accelerator, or function");
			    yyval.function.func = F_NOP;
			    yyval.function.arg = "";
			;
    break;}
case 120:
#line 1062 "mwmparse.y"
{ yyval.string = yyvsp[0].string; ;
    break;}
case 121:
#line 1064 "mwmparse.y"
{ yyval.string = ""; ;
    break;}
case 122:
#line 1068 "mwmparse.y"
{ yyval.string = yyvsp[0].string; ;
    break;}
case 123:
#line 1070 "mwmparse.y"
{ yyval.string = "ROOT"; ;
    break;}
case 124:
#line 1072 "mwmparse.y"
{ yyval.string = "WINDOW"; ;
    break;}
case 125:
#line 1074 "mwmparse.y"
{ yyval.string = "TRANSIENT"; ;
    break;}
case 126:
#line 1076 "mwmparse.y"
{ yyval.string = "ICON"; ;
    break;}
case 127:
#line 1078 "mwmparse.y"
{ yyval.string = "WITHIN"; ;
    break;}
case 128:
#line 1080 "mwmparse.y"
{ yyval.string = "FREE_FAMILY"; ;
    break;}
case 129:
#line 1082 "mwmparse.y"
{ yyval.string = "NEXT"; ;
    break;}
case 130:
#line 1084 "mwmparse.y"
{ yyval.string = "PREV"; ;
    break;}
case 131:
#line 1086 "mwmparse.y"
{ yyval.string = "BACK"; ;
    break;}
case 132:
#line 1089 "mwmparse.y"
{ yyval.string = yyvsp[0].string; ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 554 "/usr/share/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;

 yyacceptlab:
  /* YYACCEPT comes here.  */
#ifndef YYSTACK_USE_ALLOCA
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
#endif
  return 0;

 yyabortlab:
  /* YYABORT comes here.  */
#ifndef YYSTACK_USE_ALLOCA
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
#endif    
  return 1;
}
#line 1091 "mwmparse.y"


/*
 * these to variables are used by the parser to control input
 */
#define MAX_UNGET	1024
static const char *curpos;
static const char *endpos;
static const char *input_buf;
static char unget[MAX_UNGET];
static int upos = 0;

static char *_MwmRootMenu = DEFAULT_MWM_ROOT_MENU;
static char *_MwmKeyBindings = DEFAULT_MWM_KEY_BINDINGS;
static char *_MwmWindowMenu = DEFAULT_MWM_WINDOW_MENU;
static char *_MwmBehaviorKey = MWM_BEHAVIOR_KEY_BINDINGS;
static char *_MwmBuiltinButtonBindings = BUILTIN_MWM_BUTTON_BINDINGS;
static char *_MwmBuiltinMenuButtonBindings = BUILTIN_MENU_BUTTON_BINDINGS;
static char *_MwmBuiltinKillButtonBindings = BUILTIN_KILL_BUTTON_BINDINGS;
static char *_MwmBuiltinIconButtonBindings = BUILTIN_ICON_BUTTON_BINDINGS;

void
yyerror(char *fmt, ...) {
    va_list arg_list;

    va_start(arg_list, fmt);
    vfprintf(stderr, fmt, arg_list);
    va_end(arg_list);
    fprintf(stderr, " at line %d\n", lineno);
}

/*
 * find the config file
 */
static Boolean
find_config_file2(const char *the_file) {
    if (verbose)
	printf("Checking for config file '%s'\n", the_file);
    if (access(the_file, R_OK) == 0)
	return True;
    return False;
}


static Boolean
find_config_file3(char *buf, const char *cfile) {
    char *lang, *ptr;

    /* Shall overrun check be done for each strcat()??? */
    strcat(buf, "/");

    lang = getenv("LANG");
    if (!lang)
	lang = "";

    ptr = buf + strlen(buf);

    if (strlen(lang) != 0) {
	strcat(buf, lang);
	strcat(buf, "/");
	strcat(buf, cfile);
	if (find_config_file2(buf)) {
	    return True;
	}
    }

    strcpy(ptr, cfile);
    if (find_config_file2(buf)) {
	return True;
    }
    return False;
}


static Boolean
find_config_file(char *buf, const char *cfile) {
    char *home;

    home = getenv("HOME");

    if (!home)
	home = getcwd(NULL, MAX_PATH_LEN);

    switch (*cfile) {
    case '/':
	if (find_config_file2(cfile)) {
	    strcpy(buf, cfile);
	    return True;
	}
	break;
    case '~':
	if (*(cfile+1) == '/') {
	    strcpy(buf, home);
	    strcat(buf, "/");
	    strcat(buf, cfile+2);
	    if (find_config_file2(buf)) {
		return True;
	    }
	}
	else {
	    /* XXX function not implemented yet */
	    if (verbose)
		printf("~user notation is not implemented.\n");
	}
	break;
    default: /* check for home dir */
	strcpy(buf, home);
	if (find_config_file3(buf, cfile))
	    return True;
    }  /* switch (*cfile) */


    strcpy(buf, home);
    if (find_config_file3(buf, HOME_MWMRC))
	return True;
#ifndef __EMX__
    strcpy(buf, MWM_DDIR);
#else
    strcpy(buf, (char*)__XOS2RedirRoot("/XFree86/lib/X11/mwm"));
#endif
    if (find_config_file3(buf, SYSTEM_MWMRC))
	return True;

    return False;
}

char
mwm_getc(void) {
    char c;

    if (upos) {
	upos--;
	c = unget[upos];
	if (c == '\n')
	    lineno++;
	return c;
    }
    if (curpos >= endpos)
	return 0;
    else if (*curpos == 0) {
	curpos++;
	return 0;
    }
    c = *curpos;
    curpos++;
    if (c == '\n')
	lineno++;
    return c;
}

void
mwm_putc(char c) {
    printf("OUTPUT: %c\n", c);
}

void
mwm_unputc(char c) {
    if (upos == MAX_UNGET) {
	yyerror("Lexer can't back up anymore.\n");
	return;
    }
    if (c == '\n')
	lineno--;
    unget[upos] = c;
    upos++;
}

int
PARSE_buf(ScreenInfo *scr, char *buffer) {
    pscr = scr;
    lineno = 1;
    curpos = input_buf = buffer;
    endpos = input_buf + strlen(input_buf);
    upos = 0;
    return yyparse();
}

static void
do_standard(ScreenInfo *scr)
{
    skip_test = True;

    PARSE_buf(scr, _MwmRootMenu);
    PARSE_buf(scr, _MwmWindowMenu);
    PARSE_buf(scr, _MwmKeyBindings);
    PARSE_buf(scr, _MwmBehaviorKey);

    /* a certain amount seems to be expected as builtin */
    PARSE_buf(scr, _MwmBuiltinButtonBindings);
    if (Mwm.w_menu_button_click)
	PARSE_buf(scr, _MwmBuiltinMenuButtonBindings);
    if (Mwm.w_menu_button_click_2)
	PARSE_buf(scr, _MwmBuiltinKillButtonBindings);
    if (Mwm.icon_click)
	PARSE_buf(scr, _MwmBuiltinIconButtonBindings);


    PROP_SetBehavior(scr, False);
}

static void
do_custom(ScreenInfo *scr)
{
    char buf[MAX_PATH_LEN], *ptr;
    int fd;
#ifdef HAVE_SYS_STAT_H
    struct stat st;
#else
#error "you lose (I don't know how to fix this)"
#endif

    ptr = Mwm.config_file;
    if (!find_config_file(buf, Mwm.config_file)) {
	yyerror("Cannot find configuration file.  "
		"Using builtin defaults.\n");

	do_standard(scr);
    }
    else if (stat(buf, &st)) {
	yyerror("Cannot stat configuration file '%s'.\n"
		"Using builtin defaults.\n", buf);

	do_standard(scr);
    }
#ifndef __EMX__
    else if ((fd = open(buf, O_RDONLY)) < 0) {
#else
    else if ((fd = open(buf, O_RDONLY|O_BINARY)) < 0) {
#endif
	yyerror("Cannot open configuration file '%s'.  "
		"Using builtin defaults.\n");

	do_standard(scr);
    }
    else {
	ptr = XtMalloc(st.st_size + 1);
	if (read(fd, ptr, st.st_size) != st.st_size) {
	    yyerror("Cannot read configuration file '%s'.  "
		    "Using builtin defaults.\n", buf);

	    do_standard(scr);
	}
	else {
	    if (verbose)
		printf("Reading configuration from '%s'\n", buf);

	    skip_test = False;
	    ptr[st.st_size] = 0;
	    PARSE_buf(scr, ptr);

	    /* a certain amount seems to be expected as builtin */
	    skip_test = True;

	    PARSE_buf(scr, _MwmBuiltinButtonBindings);
	    if (Mwm.w_menu_button_click)
		PARSE_buf(scr, _MwmBuiltinMenuButtonBindings);
	    if (Mwm.w_menu_button_click_2)
		PARSE_buf(scr, _MwmBuiltinKillButtonBindings);
	    if (Mwm.icon_click)
		PARSE_buf(scr, _MwmBuiltinIconButtonBindings);

	    PROP_SetBehavior(scr, True);
	}

	XtFree(ptr);
    }
}

void
PARSE_mwmrc(ScreenInfo *scr) {
    int flag;

    key_bind_found = False;
    button_bind_found = False;
    key_bind_match = False;
    button_bind_match = False;

    if ((flag = PROP_GetBehavior(scr)) & MWM_INFO_STARTUP_CUSTOM) {
	if (verbose)
	    printf("parsing custom file.\n");
	do_custom(scr);
    }
    else if (flag & MWM_INFO_STARTUP_STANDARD) {
	if (verbose)
	    printf("parsing standard bindings.\n");
	do_standard(scr);
    }
    else {
	if (verbose)
	    printf("parsing custom file by default.\n");
	do_custom(scr);
    }

    return;
}
